SystemOrganization addCategory: #'System-Announcements'!

Announcer subclass: #SystemAnnouncer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

SystemAnnouncer class
	instanceVariableNames: 'announcer privateAnnouncer'!

!SystemAnnouncer commentStamp: '<historical>' prior: 0!
This class plays the role of the announcer for events raised by the Pharo system.

Currently, it acts as a wrapper over the SystemChangeNotifier, but in the future it should be used for announcing the events directly from the system.. 

BEWARE: You should not need to subclass me. Think about just using me to send your announces (if you need system behavior) or using your own announcers as instance or class  variables.!

SystemAnnouncer class
	instanceVariableNames: 'announcer privateAnnouncer'!

!SystemAnnouncer class methodsFor: 'private' stamp: 'StephaneDucasse 10/12/2011 19:39'!
announceFromEvent: aChangeEvent
	"This method creates an announcement from a given Event (subclass of AbstractEvent) and announces it.
	
	For example, from an AddedEvent that concerns a class the method creates SystemAddedClassAnnouncement and then it will announce it both from the generic announcer and from the classAnnouncer "
	| announcement |
	announcement := aChangeEvent asSystemAnnouncement.
	self privateAnnouncer announce: announcement.		
	self announcer announce: announcement.! !

!SystemAnnouncer class methodsFor: 'accessing' stamp: 'EstebanLorenzano 5/21/2012 17:26'!
announcer
	"Answer a system annoncer."
	^ self current! !

!SystemAnnouncer class methodsFor: 'accessing' stamp: 'GuillermoPolito 4/27/2012 13:25'!
announcer: anAnnouncer
	announcer := anAnnouncer! !

!SystemAnnouncer class methodsFor: 'accessing' stamp: 'EstebanLorenzano 5/21/2012 17:26'!
current
	"Answer a system annoncer."
	
	^ announcer ifNil: [ announcer := self new ]! !

!SystemAnnouncer class methodsFor: 'utilities' stamp: 'StephaneDucasse 10/12/2011 14:57'!
defaultPreferredAnnouncementClassForEvent: aChangeEvent

	| itemKindName eventKindName announcementClassName announcementClass |
	itemKindName := aChangeEvent itemKind.
	eventKindName := (aChangeEvent class name endsWith: 'Event') 
		ifTrue: [ aChangeEvent class name copyFrom: 1 to: aChangeEvent class name size - 'Event' size ] 
		ifFalse: [ aChangeEvent class name ].
	announcementClassName := 	(
										'System', 
										itemKindName capitalized, 
										eventKindName,
										'Announcement'
									) asSymbol.
	announcementClass := Smalltalk at: announcementClassName ifAbsent: [nil].
	^ announcementClass! !

!SystemAnnouncer class methodsFor: 'initialization' stamp: 'StephaneDucasse 10/12/2011 23:05'!
initialize
	"self initialize"
	
	SystemChangeNotifier uniqueInstance 
		notify: self ofAllSystemChangesUsing: #announceFromEvent:! !

!SystemAnnouncer class methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 14:56'!
privateAnnouncer
	"Answer a system annoncer."

	^ privateAnnouncer ifNil: [ privateAnnouncer := Announcer new ]! !

!SystemAnnouncer class methodsFor: 'initialization' stamp: 'lr 2/22/2010 11:10'!
unload
	SystemChangeNotifier uniqueInstance 
		noMoreNotificationsFor: self! !

!SystemAnnouncer class methodsFor: 'utilities' stamp: 'StephaneDucasse 10/16/2011 18:52'!
unsubscribe: anObject
	"unsubscribe an object from my public/private announcers"
	
	self current unsubscribe: anObject.
	self privateAnnouncer unsubscribe: anObject! !

!ReorganizedEvent methodsFor: '*System-Announcements' stamp: 'StephaneDucasse 7/15/2010 23:19'!
category

	^self itemCategory! !

!ModifiedMethodEvent methodsFor: '*System-Announcements' stamp: 'HenrikSperreJohansen 1/13/2012 14:29'!
asSystemAnnouncement
	^ MethodModified new event: self! !

!AbstractEvent methodsFor: '*System-Announcements' stamp: 'HenrikSperreJohansen 1/13/2012 14:34'!
asSystemAnnouncement
	| itemKindName eventKindName announcementClassName announcementClass |
	itemKindName := self itemKind.
	eventKindName := (self class name endsWith: 'Event') 
		ifTrue: [ self class name copyFrom: 1 to: self class name size - 'Event' size ] 
		ifFalse: [ self class name ].
	announcementClassName := 	( 
										itemKindName capitalized, 
										eventKindName
										
									) asSymbol.
	announcementClass := Smalltalk at: announcementClassName ifAbsent: [nil].
	^ announcementClass isNil 
		ifTrue: [ UnknownSystemChange new ]
		ifFalse: [announcementClass new event: self]! !

Announcement subclass: #SystemAnnouncement
	instanceVariableNames: 'event timeStamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!SystemAnnouncement commentStamp: '<historical>' prior: 0!
The root announcement for the events coming from the Pharo system.

Currently, it wraps the event that comes from the SystemChangeNotifier.!

SystemAnnouncement subclass: #CategoryAdded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!CategoryAdded commentStamp: 'cyrilledelaunay 1/18/2011 12:29' prior: 0!
This announcement will be emited when adding a category using:
=> SystemOrganizer >> addCategory:!

!CategoryAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:12'!
categoryName

	^ self item! !

!CategoryAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 7/28/2010 10:34'!
item
	^ event item! !

SystemAnnouncement subclass: #CategoryRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!CategoryRemoved commentStamp: 'cyrilledelaunay 1/18/2011 12:28' prior: 0!
This announcement will be emited when removing a category using:
=> SystemOrganizer >> removeCategory:!

!CategoryRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:12'!
categoryName

	^ self  item! !

!CategoryRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:12'!
categorySelector

	^ self item asSymbol! !

SystemAnnouncement subclass: #CategoryRenamed
	instanceVariableNames: 'from to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!CategoryRenamed commentStamp: 'cyrilledelaunay 1/18/2011 12:26' prior: 0!
This announcement will be emited when renaming a category using:
=> SystemOrganizer >> renameCategory:toBe:!

!CategoryRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 7/27/2010 21:59'!
newName

	^ event newName! !

!CategoryRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 7/27/2010 21:59'!
oldName

	^ event oldName! !

!CategoryRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:14'!
oldNameSymbol

	^ event oldName asSymbol! !

SystemAnnouncement subclass: #ClassAdded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ClassAdded commentStamp: 'cyrilledelaunay 1/18/2011 11:56' prior: 0!
This announcement will be emitted when a class or a trait is added, using:
	=> Trait >> named: (the notification is done in Trait >> named:uses:category:env:)
	=> Class >> subclass:
	!

!ClassAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:15'!
itemClass

	^ self item ! !

!ClassAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:15'!
itemClassSymbol

	^ self itemClass theNonMetaClass name asSymbol! !

SystemAnnouncement subclass: #ClassCommented
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

SystemAnnouncement subclass: #ClassModifiedClassDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

SystemAnnouncement subclass: #ClassRecategorized
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ClassRecategorized commentStamp: 'cyrilledelaunay 1/18/2011 10:42' prior: 0!
This announcement will be raised when:
- we set a category to a class (see 'Class >>category:')
- we set a category to a Trait (see Trait>>category:)

It corresponds to the RecategorizedEvent!

!ClassRecategorized methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 7/27/2010 22:38'!
itemClass 

	^ event itemClass ! !

!ClassRecategorized methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 7/28/2010 09:09'!
newCategory 

	^ event itemCategory ! !

!ClassRecategorized methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 4/24/2011 18:24'!
oldCategory 

	^ event oldCategory asSymbol! !

SystemAnnouncement subclass: #ClassRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ClassRemoved commentStamp: 'cyrilledelaunay 1/18/2011 11:43' prior: 0!
the annoucement will be emitted when removing a class or a trait using:  
	=> removeFromSystem!

!ClassRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 5/26/2012 16:15'!
removedClass

	^ self item theNonMetaClass! !

!ClassRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 5/26/2012 16:10'!
removedClassCategory

	"the previous definition was self item class theNonMetaClass category
	but his makes the assumption that the class is a living and well connected object"
	
	^ self event itemCategory! !

SystemAnnouncement subclass: #ClassRenamed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ClassRenamed commentStamp: 'cyrilledelaunay 1/18/2011 11:44' prior: 0!
the annoucement will be emitted when renaming a class or a trait using:  
	=> RenameClassRefactoring >> rename:to:
	=> class>>rename:
The corresponding event is raised in: SystemDictionary>>renameClass:from:to:!

!ClassRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:16'!
itemClass

	^ self item! !

!ClassRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:16'!
itemPackageName
	"It cannot be self itemClass package name since itemClass refer to the new class which is not in
	the current RPackageOrganizer - because we are currently in the process of rename it.
	
	It cannot be  (Smalltalk globals at: self oldName) package name because oldName class is not anymore in Smalltalk globals."
	^ self event itemCategory
	! !

!ClassRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:16'!
newName
	
	^ self event newName! !

!ClassRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:16'!
oldName
	
	^ self event oldName! !

SystemAnnouncement subclass: #ClassReorganized
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ClassReorganized commentStamp: 'cyrilledelaunay 1/18/2011 15:03' prior: 0!
This announcement corresponds to the ReorganizedEvent, which seems to (by looking at the references of ReorganizedEvent) be raised when:
- we rename a protocol (see 'renameCategory:toBe: '). if The category is not empty, SystemMethodRecategorizedAnnouncement will also be emitted
- we sort (modify the order of) protocols (see 'sortCategories')
- we add a protocol (see 'addCategory:before:')
- we remove a protocol (see 'removeCategory:' 'removeEmptyCategories')!

!ClassReorganized methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:16'!
itemClass

	^ self item! !

SystemAnnouncement subclass: #MethodAdded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!MethodAdded commentStamp: 'cyrilledelaunay 1/18/2011 13:06' prior: 0!
This announcement is emited when we add a method to a class or a trait using:
	=> Behavior >> compile: or TraitBehavior >> compile:!

!MethodAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 7/16/2010 13:08'!
itemClass

	^ event itemClass
	! !

!MethodAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:17'!
itemMethod

	^ self item! !

!MethodAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 7/16/2010 13:08'!
itemProtocol

	^ event itemProtocol! !

!MethodAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 7/16/2010 13:08'!
itemSelector

	^ event itemSelector
	! !

SystemAnnouncement subclass: #MethodModified
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!MethodModified commentStamp: 'BenjaminVanRyseghem 4/1/2011 16:04' prior: 0!
This announcement is emited when we RE-compile a method in a class or a trait, with: ClassDescription >> compile: or TraitDescription >> compile:. If the method is not yet registered in the class or the trait, the announcement will not be emitted.

The action of renaming a method will be handled by SystemMethodRemovedAnnouncement and SystemMethodAddedAnnouncement, since this refactoring is concretely composed by removing the old method and add a new with the new name  !

!MethodModified methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:17'!
item

	^ event item! !

!MethodModified methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/17/2011 14:45'!
itemClass

	^ event itemClass
	! !

!MethodModified methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/18/2011 13:50'!
itemNewProtocol

	^ event newProtocol! !

!MethodModified methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/18/2011 13:50'!
itemOldProtocol

	^ event oldProtocol! !

!MethodModified methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/17/2011 14:44'!
itemProtocol

	^ event itemProtocol! !

!MethodModified methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/17/2011 14:45'!
itemSelector

	^ event itemSelector
	! !

!MethodModified methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:17'!
newMethod

	^ event oldItem! !

!MethodModified methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:17'!
oldItem

	^ event oldItem! !

!MethodModified methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:17'!
oldMethod
	
	^ self  item! !

SystemAnnouncement subclass: #MethodRecategorized
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!MethodRecategorized commentStamp: 'cyrilledelaunay 1/18/2011 15:03' prior: 0!
This announcement will be emitted when:
	=> a non-empty protocol is renamed in a class or a trait (using ClassOrganizer >> renameCategory:toBe:). If the protocol does not contain any method, the announcement will not be emitted. Therefore, if we rename a non-empty protocol, both SystemMethodRecategorizedAnnouncement and SystemClassReorganizedAnnouncement will be emitted
	=> a method is removed from a category of a class or a trait (using ClassOrganizer >> removeElement:)
	=> a method is RE-categorized in a protocol of a trait or a class, using Organizer>>classify:under:.  If the method is classify under its current protocol, the announcement will not be emitted!

!MethodRecategorized methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/17/2011 15:40'!
itemClass

	^ event itemClass
	! !

!MethodRecategorized methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/17/2011 16:14'!
itemNewProtocol

	^ event itemProtocol! !

!MethodRecategorized methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/17/2011 16:11'!
itemOldProtocol

	^ event oldCategory! !

!MethodRecategorized methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/17/2011 15:41'!
itemSelector

	^ event itemSelector
	! !

SystemAnnouncement subclass: #MethodRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!MethodRemoved commentStamp: 'cyrilledelaunay 1/18/2011 14:43' prior: 0!
This announcement will be emitted when a method is removed from a trait or a class, 
using ClassDescription >>removeSelector: or TraitDescription >> removeSelector.
!

!MethodRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 4/13/2011 21:23'!
itemClass

	^ self event itemClass
	! !

!MethodRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:18'!
itemMethod

	^ event item
	! !

!MethodRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 4/13/2011 21:23'!
itemProtocol

	^ self event itemProtocol! !

!MethodRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 4/13/2011 21:23'!
itemSelector

	^ self event itemSelector
	! !

SystemAnnouncement subclass: #ProtocolAdded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ProtocolAdded commentStamp: '<historical>' prior: 0!
This class is not used, but should be used when we hook into the addition and removal of protocols. Right now, we only get ClassReorganizedAnnouncement!

SystemAnnouncement subclass: #ProtocolRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ProtocolRemoved commentStamp: '<historical>' prior: 0!
This class is not used, but should be used when we hook into the addition and removal of protocols. Right now, we only get ClassReorganizedAnnouncement!

!SystemAnnouncement class methodsFor: 'instance creation' stamp: 'StephaneDucasse 4/13/2011 17:03'!
fromInternal: anInternalAnnouncement
	"create a new announcement from an internal announcement. Copy the variables"
	
	| instance |
	self flag: #ben. "why copy does not work?"
	instance := self new.
	1 to: self allInstVarNames size do:[:i | instance instVarAt: i put: ( anInternalAnnouncement instVarAt: i )].
	^ instance! !

!SystemAnnouncement class methodsFor: 'instance creation' stamp: 'StephaneDucasse 8/9/2011 23:05'!
new 
	
	^ super new setTimeStamp; yourself! !

!SystemAnnouncement methodsFor: 'private' stamp: 'tg 6/5/2010 12:17'!
event
	^ event! !

!SystemAnnouncement methodsFor: 'private' stamp: 'tg 6/5/2010 12:17'!
event: anObject
	event := anObject! !

!SystemAnnouncement methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 4/1/2011 17:18'!
item

	^ self event item! !

!SystemAnnouncement methodsFor: 'private' stamp: 'StephaneDucasse 8/9/2011 23:05'!
setTimeStamp

	timeStamp := TimeStamp current.! !

!SystemAnnouncement methodsFor: 'accessing' stamp: 'StephaneDucasse 8/9/2011 23:04'!
timeStamp

	^ timeStamp! !

!SystemAnnouncement methodsFor: 'accessing' stamp: 'StephaneDucasse 8/9/2011 23:04'!
timeStamp: aTimeStamp

	timeStamp := aTimeStamp! !

SystemAnnouncement subclass: #UnknownSystemChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

Object subclass: #AnnouncementLogger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!AnnouncementLogger commentStamp: 'StephaneDucasse 5/9/2011 17:17' prior: 0!
Use me to debug and log to the transcript annoncements.


AnnouncementLogger new subscribeTo: SystemAnnouncer announcer

To unsubscribe

AnnouncementLogger  allInstancesDo: [:each | SystemAnnouncer announcer unsubscribe: each ]!

!AnnouncementLogger methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 5/9/2011 17:11'!
logAnnouncement: ann

	Transcript show: ann printString ; cr! !

!AnnouncementLogger methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 5/9/2011 17:15'!
subscribeTo: anAnnouncer

	anAnnouncer "weak" 
		on: Announcement send: #logAnnouncement: to: self! !
SystemAnnouncer initialize!
